/*********************************************************************
 *
 * File:         application.c
 *
 * Description:  NPAC Conferencing System API functions for applications 
 *               management 
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-22-96
 *
 *********************************************************************/
#include <stdio.h>
#include <winsock.h>

#include "common.h"
#include "rtp.h"
#include "multicast.h"
#include "database.h"

FILE *alog;

void ncsReadAppMessage(ncsApplication *app);

int multicast = 0;
int database = 0;

//extern int errno;



/************************************************************************** 
 *
 * Conference Engine messages callbacks
 *
 **************************************************************************/
void appQuit(ncsApplication *app)
{
  if((multicast) && (app->session->localMember >= 0))
    closeMulticast(app->socket);
  closesocket(app->socket);
  free(app);
}


/************************************************************************** 
 *
 * API functions 
 *
 **************************************************************************/
ncsApplication *ncsAppAttach(int port, Session *ses)
{
  ncsApplication *app;

#ifndef QUIET
  alog = fopen("app.log", "w");
#endif

  app = (ncsApplication *)malloc(sizeof(ncsApplication));
  
  app->port = port;
  app->session = ses;
  
  /* Initialize application data socket */
  if ((app->socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      PRINTF(alog, "Application: can't open socket\n");
      free(app);
      return NULL;
    }
  
  app->addr.sin_family = AF_INET;
  app->addr.sin_addr.s_addr = htonl(INADDR_ANY);
  app->addr.sin_port        = htons(port);
  
  if(bind(app->socket, (struct sockaddr *)&app->addr, sizeof(struct sockaddr)) < 0)
    {
      PRINTF(alog, "Application: can't bind socket\n");
      free(app);
      return NULL;
    }

  return app;
}

void ncsAppDettach(ncsApplication *app)
{
  if(app != NULL)
    {
      appQuit(app);

      if(database)
		{
			dbClose();
		}
    }
  
#ifndef QUIET
  fclose(alog);
#endif

}

int ncsAppSendData(ncsApplication *app, int mbr, int type, void *data, int len)
{
  char *ncsdata;
  char *tmp;
  int size;
  int i;
  int sent = 0;
  RTPheader header;

  size = len + RTP_HEADER_SIZE;
  ncsdata = (char *)malloc(size);
  RTPfillHeader(&header, type);
  tmp = ncsdata + RTP_HEADER_SIZE;
  memcpy(tmp, data, len);
  header.ssrc = app->session->localMember;
  fillRTPheader(&header, ncsdata);

  if(mbr < 0)  /* send to all members */
  {
	if(multicast)
	{
		if(sendto(app->socket, (char *)ncsdata, size, 
			0, (struct sockaddr *)&app->mcast_addr, sizeof(struct sockaddr_in)) < 0)
		return 0;  
	}
    else
	{
		for(i=0; i<MAX_MEMBERS; i++)
			if(app->session->memberActive[i])
			{
				if(i != app->session->localMember)
				{
					PRINTF(alog, "Application: data sent to user %d, size %d\n", i, size);
					sendPacket(app->socket, app->session->member[i].host, app->port, (char *)ncsdata, size);
				}
			}
	}

  }
  else   /* send to one specific member */
  {
	if(app->session->memberActive[mbr])
		if(mbr != app->session->localMember)
		{
			sendPacket(app->socket, app->session->member[mbr].host, app->port, (char *)ncsdata, size);
		}

	/* if message to initiator find initiator */
    if((mbr == 0) && (app->session->memberActive[mbr] == 0))
	{
		i = 0;
		while((i < MAX_MEMBERS) && (!sent))
	    {
			if(i != app->session->localMember)
			{
				PRINTF(alog, "Application: data sent to user %d, size %d\n",i, size);
				sendPacket(app->socket, app->session->member[i].host, app->port, (char *)ncsdata, size);
				sent = 1;
			}
			i++;
	    }
	}
  }      

  free(ncsdata);

  return 1;
}

// TO DO: change this procedure
int ncsAppSendPriorityData(ncsApplication *app, int mbr, int type, void *data, int len)
{
/*
	AppData *appdata;
  int size;

  size = sizeof(AppData) + len - 1;
  appdata = (AppData *)malloc(size);
  appdata->handle = app->localHandle;
  appdata->recID = mbr;
  appdata->port = app->port;
  appdata->tag = type;
  memcpy(appdata->data, data, len);

  appEngineRequest(app, reqAppData, (char *)appdata, size);

  free(appdata);
*/

  return 1;
}

/*************************************************************************
 * 
 * Proccesses the NCS message incoming from the application control  
 * socket (from the Conference Engine) 
 *
 *************************************************************************/ 
int ncsAppProcessIncoming(ncsApplication *app, fd_set *set)
{
  return 1;
}

/*************************************************************************
 * 
 * Receive the application data(if any) from the application data  
 * socket (from remote applications) 
 *
 *************************************************************************/ 
int ncsAppGetData(ncsApplication *app, fd_set *set, char **data)
{
  int n, remote_addr_len;
  struct sockaddr_in remote_addr;

  remote_addr_len = sizeof(struct sockaddr_in);
  if(FD_ISSET(app->socket, set))
    {
      PRINTF(alog, "Application: there are data\n");

      *data = (char *)malloc(MAX_UDP_SIZE);
      n = recvfrom(app->socket, (char *)*data, MAX_UDP_SIZE, 
		   0, (struct sockaddr *)&remote_addr, &remote_addr_len); 
      if (n < 0)
	{
	  PRINTF(alog, "Application: recvfrom error\n");
	  return 0;
	}

      if(database)
	{
	  dbAdd(*data, n);
	}
    }
  else
    return 0;

  return n;
}  


/***********************************************************************
 * 
 * Get the last file descriptor used by NCS and set the file descriptors
 * set 'set' for all descriptors used by NCS
 *
 ***********************************************************************/
int ncsAppGetFD(ncsApplication *app, fd_set *set)
{
  FD_ZERO(set);
  FD_SET(app->socket, set);
  return app->socket;
}

int ncsAppGetLocalHandle(ncsApplication *app)
{
  return app->session->localMember;
}

int ncsAppGetNumMembers(ncsApplication *app)
{
  return app->session->numMembers;
}

	
int ncsAppGetPart(ncsApplication *app, int no, char *name, char *host)
{
	if (app->session->memberActive[no] == 1)
	{
		strcpy(name, app->session->member[no].user);
		ncsGetHostName(app->session->member[no].host, host);
	}
	else
	{
		strcpy(name, "unknown");
		strcpy(host, "");
		return 0;
	}

	return 1;
}

void readRTPheader(RTPheader *hdr, char *data)
{
	hdr->type = (data[0] & 192) >> 6;
	hdr->p = (data[0] & 32) >> 5;
	hdr->x = (data[0] & 16) >> 4;
	hdr->cc = data[0] & 15;
	hdr->m = (data[1] & 128) >> 7;
	hdr->pt = data[1] & 127;
	hdr->seq = ntohs(*((short *)&data[2]));
	hdr->ts = ntohl(*((int *)&data[4]));
	hdr->ssrc = ntohl(*((int *)&data[8]));
}

void fillRTPheader(RTPheader *hdr, char *data)
{
	short tmp1;
	int tmp2;
	int i;

	for(i=0; i<RTP_HEADER_SIZE; i++)
		data[i] = 0;
	
	data[0] = data[0] | ((hdr->type << 6) & 192);
	data[0] = data[0] | ((hdr->p << 5) & 32);
	data[0] = data[0] | ((hdr->x << 4) & 16);
	data[0] = data[0] | (hdr->cc & 15);
	data[1] = data[1] | ((hdr->m << 7) & 128);
	data[1] = data[1] | (hdr->pt & 127);
	tmp1 = htons(hdr->seq);
	memcpy(&data[2], &tmp1, 2);
	tmp2 = htonl(hdr->ts);
	memcpy(&data[4], &tmp2, 4);
	tmp2 = htonl(hdr->ssrc);
	memcpy(&data[8], &tmp2, 4);
}











