/*********************************************************************
 *
 * File:         session_manage.c
 *
 * Description:  Session management module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"
#include "session_manage.h"

extern char localHost[];   /* local host name */
Session session;

/************************************************************************
 *
 * Additional procedures
 *
 ************************************************************************/
int addr2name(struct sockaddr_in *addr, char *name)
{
  struct hostent *hostinfo;

  hostinfo = gethostbyaddr(&addr->sin_addr, sizeof(struct in_addr), AF_INET);
  if(hostinfo == NULL)
    {
      fprintf(stderr, "Engine: new member unknown host\n");
      return 0;
    }
  strcpy(name, hostinfo->h_name);
  return 1;
}


int start_process(char *path)
{
  int pid;

  if((pid = fork()) < 0)
    {
      fprintf(stderr, "Engine: fork error\n");
      return -1;
    }
  else if(pid == 0)
    {
      execl(path, 0);
    }
  else
    return pid;
}

void startCM()
{
  char *ncs_path;
  char path[100];
  int pid;

  ncs_path = getenv("NCS_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/CM/NCSmanager");

  start_process(path);
}

int newMember(Participant *member)
{
  MemberInfo info;
  int pos;

  pos = member->handle;
  session.numMembers++;
  session.memberActive[pos] = 1;
  session.member[pos].handle = pos;
  strcpy(session.member[pos].user, member->user);
  session.member[pos].host = member->host;

  sendLocalMessage(indLocalSessionAddMember, 
		     (char *)member, sizeof(Participant));

  sendAppMessage(-1, indLocalSessionAddMember, (char *)member, sizeof(Participant));
  return 1;
}

/************************************************************************
 *
 * Initialization and creation
 *
 ************************************************************************/
int sessionInit()
{
  int i;

  session.status = statusInit;
  session.lastMember = -1;
  session.numMembers = 0;
  session.localMember = -1;
  for(i=0; i<MAX_MEMBERS; i++)
    session.memberActive[i] = 0;

  for(i=0; i<MAX_APPS; i++)
    session.appActive[i] = 0;
  session.numApps = 0;
  session.lastApp = -1;
}

void setSessionID(int id)
{
  session.id = id;
}

int sessionCreate(Participant *initiator)
{
  if(session.status == statusInit)
    {
      PRINTF("Engine: session initiator %s@%d\n", initiator->user, initiator->host);
      session.status = statusCreated;

      newMember(initiator);
      return 1;
    }
  else
    {
      PRINTF("Engine: session already created -- status %d\n", session.status);
      return 0;
    }
}

void sessionAbandon()
{
  sessionInit();
  session.status = statusAbandoned;
}

/************************************************************************
 *
 * Members management
 *
 ************************************************************************/
int sessionAddMember(Participant *newmember)
{
  int current;

  if(session.status == statusCreated)
    {
      if(session.lastMember < MAX_MEMBERS-1)
	{
	  if(getUserHandle(newmember) == -1)
	    {
	      PRINTF("Engine: session member %s@%d\n", 
		     newmember->user, newmember->host);
	      newMember(newmember);
	      return 1;
	    }
	  else 
	    return 0;
	}
    }
  return 0;
}

void sessionDeleteMember(int no)
{
  session.memberActive[no] = 0;
  session.numMembers--;
  sendLocalMessage(indLocalSessionDeleteMember, 
		   (char *)&no, sizeof(int));
  sendAppMessage(-1, indLocalSessionDeleteMember, (char *)&no, sizeof(int));
}


void sessionSetLocalMember(int no)
{
  PRINTF("Engine: session set local member %d\n", no);
  session.localMember = no;
  sendAppMessage(-1, indLocalSessionLocalMember, 
	     (char *)&session.localMember, sizeof(int));
}

/************************************************************************
 *
 * Applications management
 *
 ************************************************************************/
int sessionAddApp(int no, struct sockaddr_in *addr)
{
  MemberInfo info;
  int i;

  if(session.numApps < MAX_APPS)
    {
      PRINTF("Engine: add application\n");
      session.numApps++;
      session.appActive[no] = 1;
      session.appPort[no] = ntohs(addr->sin_port);
      memcpy(&session.appAddr[no], addr, 
	     sizeof(struct sockaddr_in)); 
      session.appAddr[no].sin_addr.s_addr = gethostid();

      for(i=0; i<MAX_MEMBERS; i++)
	{
	  if(session.memberActive[i])
	    {
	      sendAppMessage(no, indLocalSessionAddMember,
			 (char *)&session.member[i], sizeof(Participant));
	    }
	}

      if(session.localMember >= 0)
	sendAppMessage(no, indLocalSessionLocalMember, 
		       (char *)&session.localMember, sizeof(int));

      return 1;
    }
  else
    return 0;
}


void sessionDeleteApp(int no)
{
  session.appActive[no] = 0;
}

/************************************************************************
 *
 * Session query
 *
 ************************************************************************/
/**********************************************************************
 * Members queries 
 */
int sessionGetNoMembers()
{
  return session.numMembers;
}


void sessionGetMembers(Participant *parts)
{
  int i, no = 0;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	{
	  memcpy(parts, &session.member[i], sizeof(Participant));
	  parts++;
	}
    }
}

int getReceiverData(int userNo, MsgHeader *header)
{
  if((session.memberActive[userNo]) && (userNo != session.localMember))
    {
      memcpy(&header->receiver, &session.member[userNo], sizeof(Participant));
      return 1;
    }
  else
    return 0;
}


int getUserData(int userNo, struct sockaddr_in *addr, Participant *part)
{
  if((session.memberActive[userNo]) && (userNo != session.localMember))
    {
      memcpy(addr, &session.memberAddr[userNo], sizeof(struct sockaddr_in));
      memcpy(part, &session.member[userNo], sizeof(Participant));
      return 1;
    }
  else
    return 0;
}

int getUserHandle(Participant *part)
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	if(   (session.member[i].host == part->host)
	   && (strcmp(session.member[i].user, part->user) == 0))
	      return i;
    }

  return -1;
}

int getLocalHandle()
{
  return session.localMember;
}

/**********************************************************************
 * Status queries 
 */
int sessionCreated()
{
  if(session.status == statusCreated)
    return 1;
  else
    return 0;
}

int sessionInitialized()
{
  if(session.status == statusInit)
    return 1;
  else
    return 0;
}

int sessionAbandoned()
{
  if(session.status == statusAbandoned)
    return 1;
  else
    return 0;
}

int getSessionID()
{
  return session.id;
}

/**********************************************************************
 * Applications queries 
 */
int getAppNumber(int port)
{
  int i=0;

  while(i<MAX_APPS)
    {
      if((session.appActive[i]) && (session.appPort[i] == port))
	return i;
      else i++;
    }
  return -1;
}

